@@ -1,8 +1,6 @@
-#ifdef _OPENMP
-#include <omp.h>
-#endif
+#include "collapse_c.h" // Needs to be first because includes OpenMP
 #include "kit.h"
-#include "collapse_c.h"
+
 
 static double NEG_INF = -1.0/0.0;
 
@@ -621,8 +619,15 @@ SEXP w_mode_string(const SEXP *restrict px, const double *restrict pw, const int
 
 // Implementations for R vectors -----------------------------------------------
 
-// Splitting this up to increase thread safety
+// https://github.com/wch/r-source/blob/trunk/src/include/Rinlinedfuns.h
+SEXP my_ScalarLogical(int x) {
+  SEXP ans = allocVector(LGLSXP, 1);
+  // SET_SCALAR_LVAL(ans, x); // Not part of the API
+  LOGICAL(ans)[0] = x;
+  return ans;
+}
 
+// Splitting this up to increase thread safety
 SEXP mode_impl_plain(SEXP x, int narm, int ret) {
   int l = length(x);
   if(l <= 1) return x;
@@ -631,7 +636,7 @@ SEXP mode_impl_plain(SEXP x, int narm, int ret) {
     case REALSXP: return ScalarReal(mode_double(REAL(x), &l, l, 1, narm, ret));
     case INTSXP:  return ScalarInteger(isFactor(x) ? mode_fct_logi(INTEGER(x), &l, l, nlevels(x), 1, narm, ret) :
                                     mode_int(INTEGER(x), &l, l, 1, narm, ret));
-    case LGLSXP: return duplicate(ScalarLogical(mode_fct_logi(LOGICAL(x), &l, l, 1, 1, narm, ret)));
+    case LGLSXP: return my_ScalarLogical(mode_fct_logi(LOGICAL(x), &l, l, 1, 1, narm, ret));
     case STRSXP: return ScalarString(mode_string(STRING_PTR(x), &l, l, 1, narm, ret));
     default: error("Not Supported SEXP Type: '%s'", type2char(TYPEOF(x)));
   }
@@ -653,7 +658,7 @@ SEXP w_mode_impl_plain(SEXP x, double *pw, int narm, int ret) {
     case REALSXP: return ScalarReal(w_mode_double(REAL(x), pw, &l, l, 1, narm, ret));
     case INTSXP:  return ScalarInteger(isFactor(x) ? w_mode_fct_logi(INTEGER(x), pw, &l, l, nlevels(x), 1, narm, ret) :
                                     w_mode_int(INTEGER(x), pw, &l, l, 1, narm, ret));
-    case LGLSXP:  return duplicate(ScalarLogical(w_mode_fct_logi(LOGICAL(x), pw, &l, l, 1, 1, narm, ret)));
+    case LGLSXP:  return my_ScalarLogical(w_mode_fct_logi(LOGICAL(x), pw, &l, l, 1, 1, narm, ret));
     case STRSXP:  return ScalarString(w_mode_string(STRING_PTR(x), pw, &l, l, 1, narm, ret));
     default: error("Not Supported SEXP Type: '%s'", type2char(TYPEOF(x)));
   }
@@ -875,7 +880,7 @@ SEXP fmodeC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
     return w_mode_impl(x, pw, asLogical(Rnarm), asInteger(Rret));
   }
   if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-  const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+  const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
   int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, nthreads = asInteger(Rnthreads);
   if(l != length(pg[1])) error("length(g) must match length(x)");
   if(isNull(o)) {
@@ -943,7 +948,7 @@ SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
       }
     } else {
       if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-      const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+      const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
       ng = INTEGER(pg[0])[0];
       int sorted = LOGICAL(pg[5])[1] == 1, *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst;
       if(nrx != length(pg[1])) error("length(g) must match nrow(x)");
@@ -1053,7 +1058,7 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
 
   // With groups
   if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-  const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+  const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
   int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, gl = length(pg[1]);
   if(l != gl) error("length(g) must match nrow(x)");
   SEXP res = PROTECT(allocVector(tx, ng * col));
